import numpy as np
from devito import Grid, SparseFunction, Function, Operator, Dimension
import matplotlib.pyplot as plt

if __name__ == '__main__':

    ## Grid and Function definitions
    shape = (5, 11)
    grid = Grid (shape=shape)
    f = Function(name='f', grid = grid)
    x = np.linspace(0., 1., shape [1])
    y = np.linspace(0., 1., shape [0])
    X, Y = np.meshgrid (x, y)
    f.data[:] = X

    ## Sparse function definitions
    npoints = 101
    coordinates = np.random.rand (npoints, 2)
    points = SparseFunction (name = 'sf', grid=grid, npoint=npoints, coordinates = coordinates)

    ## Interpolating to sparse grid
    eqnInt = points.interpolate (f)
    opInt = Operator (eqnInt)
    opInt ()

    # Plotting
    fig = plt.figure ()
    ax = fig.gca (projection='3d')
    ax.plot_wireframe(X, Y, f.data)
    ax.scatter (coordinates [:, 1], coordinates [:, 0], points.data, c = 'red')
    plt.show ()

    ## Injecting from sparse grid
    fInj = Function (name='fInj', grid=grid)
    eqnInj = points.inject (fInj, points)
    opInj = Operator (eqnInj)
    opInj ()

    # Plotting
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.scatter(coordinates[:, 1], coordinates[:, 0], points.data, c='red')
    ax.plot_wireframe(X, Y, fInj.data)
    plt.show ()
